package compute

import (
	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/resource_model"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type computeMongoAccessor struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
}

// New creates a new instance of the computeMongoAccessor
func New() *computeMongoAccessor {
	return &computeMongoAccessor{}
}

/*
* Nothing special here, just the basic CRUD operations
 */

func (dca *computeMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return dca.GenericDeleteOne(id, dca)
}

func (dca *computeMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	set.(*ComputeResource).ResourceModel = nil
	return dca.GenericUpdateOne(set, id, dca, &ComputeResource{})
}

func (dca *computeMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	data.(*ComputeResource).ResourceModel = nil
	return dca.GenericStoreOne(data, dca)
}

func (dca *computeMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return dca.computeMongoAccessor(data, dca)
}

func (dca *computeMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var compute ComputeResource

	res_mongo, code, err := mongo.MONGOService.LoadOne(id, dca.GetType())
	if err != nil {
		dca.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}

	res_mongo.Decode(&compute)
	accessor := (&resource_model.ResourceModel{}).GetAccessor(nil)
	resources, _, err := accessor.Search(nil, dca.GetType())
	if err == nil && len(resources) > 0 {
		compute.ResourceModel = resources[0].(*resource_model.ResourceModel)
	}
	return &compute, 200, nil
}

func (wfa computeMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []ComputeResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	accessor := (&resource_model.ResourceModel{}).GetAccessor(nil)
	resources, _, err := accessor.Search(nil, wfa.GetType())
	for _, r := range results {
		if err == nil && len(resources) > 0 {
			r.ResourceModel = resources[0].(*resource_model.ResourceModel)
		}
		objs = append(objs, &r) // only get the abstract resource !
	}
	return objs, 200, nil
}

func (wfa *computeMongoAccessor) Search(filters *dbs.Filters, search string) ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	if (filters == nil || len(filters.And) == 0 || len(filters.Or) == 0) && search != "" {
		filters = &dbs.Filters{
			Or: map[string][]dbs.Filter{ // filter by like name, short_description, description, owner, url if no filters are provided
				"abstractresource.abstractobject.name": {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.short_description":   {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.description":         {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.owner":               {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.source_url":          {{Operator: dbs.LIKE.String(), Value: search}},
			},
		}
	}
	res_mongo, code, err := mongo.MONGOService.Search(filters, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not store to db. Error: " + err.Error())
		return nil, code, err
	}
	var results []ComputeResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	accessor := (&resource_model.ResourceModel{}).GetAccessor(nil)
	resources, _, err := accessor.Search(nil, wfa.GetType())
	for _, r := range results {
		if err == nil && len(resources) > 0 {
			r.ResourceModel = resources[0].(*resource_model.ResourceModel)
		}
		objs = append(objs, &r) // only get the abstract resource !
	}
	return objs, 200, nil
}
